import DaReManager 0.1 as DRM
import "../account_group" as AG
import "../account" as A
import "./" as SELF

DRM.FormCard {
    short_name: "Income or Expense"
    full_name: "Income or Expense"

    reload_block: SELF.DBTable{state: "select_record"}

    content: [
        DRM.FormStringSentence {caption.value:"ID";value.field:"id"; enabled.value:false }

        , DRM.FormDateSentence {caption.value:"Date";value.field:"date"}

        , DRM.FormBoolSentence {
            caption.value:"State";
            value.field:"account_income_expense";

            value_description_checked:"Income"
            value_description_unchecked:"Expense"
        }

        , DRM.FormReferenceSentence {
            caption.value:"Group"; value.field:"account_group_id";

            select_block: AG.FormRegistry{
                registry.search_on: true;

                confirm_visible: true;
                confirm_shortname: 'Select';

                output_fields_on: true;
                output_fields:[DRM.ReplaceField{source_code:'id'; code: 'account_group_id' }]

            }

            value_description.field:'group_name'

            value.reload_block: AG.DBTable {
                input_fields_on: true;
                input_fields: [DRM.ReplaceField{code: 'account_group_id'}]

                state: "sql"
                sql: "SELECT name AS group_name FROM account_group WHERE id=:account_group_id"
            }

        }

        , DRM.FormReferenceSentence {
            caption.value:"Account"; value.field:"account_id";

            select_block: A.FormRegistry{
                registry.search_on: true;

                confirm_visible: true;
                confirm_shortname: 'Select';

                output_fields_on: true;
                output_fields:[DRM.ReplaceField{source_code:'id'; code: 'account_id' }]

            }

            value_description.field:'account_name'

            value.reload_block: AG.DBTable {
                input_fields_on: true;
                input_fields: [DRM.ReplaceField{code: 'account_id'}
                               , DRM.ReplaceField{code: 'price_enabled'}
                              ]

                state: "sql"
                sql: "SELECT
                        acc.group_id AS account_group_id
                        , acc.income_expense AS account_income_expense
                        , acc.name AS account_name
                        , acc.measure AS account_measure
                        , CASE acc.measure WHEN '' THEN 'false' ELSE :price_enabled END AS price_enabled
                        , CASE acc.measure WHEN '' THEN 'false' ELSE 'true' END AS count_enabled
                    FROM account AS acc
                    WHERE acc.id=:account_id"
            }

        }

        , DRM.FormStringSentence {caption.value:"Description";value.field:"description"; }

        , DRM.FormNumberSentence {caption.value:"Price";

            enabled.visible: true;
            enabled.value: false;
            enabled.field: 'price_enabled';
            enabled.reload_block:SELF.DBTable{
                input_fields_on: true;
                input_fields: [DRM.ReplaceField{code:'price_enabled'}]
                state: 'sql'
                sql: "SELECT CASE :price_enabled WHEN 'true' THEN 'false' ELSE 'true' END AS sum_enabled"
            }

            value.field:"price";
            value.decimals: 2
            value.reload_block: SELF.DBTable{
                input_fields_on: true;
                input_fields: [DRM.ReplaceField{code:'sum'}
                               ,DRM.ReplaceField{code:'count'}
                               ,DRM.ReplaceField{code:'price'}
                               ,DRM.ReplaceField{code:'price_enabled'}
                              ]
                state: 'sql'
                sql: "SELECT CASE :price_enabled
                                WHEN 'true' THEN :price*:count
                                ELSE :sum
                            END AS sum"

            }
        }

        , DRM.FormNumberSentence {caption.value:"Count";value.field:"count";

            enabled.visible: false;
            enabled.value: true;
            enabled.field: 'count_enabled';

            value.decimals: 4;
            measure.field:"account_measure"
            value.reload_block: SELF.DBTable{
                input_fields_on: true;
                input_fields: [DRM.ReplaceField{code:'sum_enabled'}
                               ,DRM.ReplaceField{code:'price'}
                               ,DRM.ReplaceField{code:'price2'; source_code:"price"}
                               ,DRM.ReplaceField{code:'count'}
                               ,DRM.ReplaceField{code:'count2'; source_code:"count"}
                               ,DRM.ReplaceField{code:'sum'}
                               ,DRM.ReplaceField{code:'price_enabled'}
                               ,DRM.ReplaceField{code:'price1'; source_code:"price"}
                               ,DRM.ReplaceField{code:'count1'; source_code:"count"}
                               ,DRM.ReplaceField{code:'sum1'; source_code:"sum"}
                              ]
                state: 'sql'
                sql: "SELECT CASE :sum_enabled
                                WHEN 'true' THEN CASE :count WHEN 0 THEN :price ELSE :sum/:count2 END
                                ELSE :price2
                                END AS price
                            , CASE :price_enabled
                                WHEN 'true' THEN :price1*:count1
                                ELSE :sum1
                                END AS sum
                    "
            }
        }

        , DRM.FormNumberSentence {caption.value:"Summa";

            enabled.visible: true;
            enabled.value: true;
            enabled.field: 'sum_enabled';
            enabled.reload_block:SELF.DBTable{
                input_fields_on: true;
                input_fields: [DRM.ReplaceField{code:'sum_enabled'}]
                state: 'sql'
                sql: "SELECT CASE :sum_enabled WHEN 'true' THEN 'false' ELSE 'true' END AS price_enabled"
            }

            value.field:"sum";
            value.decimals: 2
            value.reload_block: SELF.DBTable{
                input_fields_on: true;
                input_fields: [DRM.ReplaceField{code:'sum'}
                               ,DRM.ReplaceField{code:'count'}
                               ,DRM.ReplaceField{code:'price'}
                               ,DRM.ReplaceField{code:'sum_enabled'}
                              ]
                state: 'sql'
                sql: "SELECT CASE :sum_enabled
                                WHEN 'true' THEN CASE :count WHEN 0 THEN :price ELSE :sum/:count END
                                ELSE :price
                            END AS price"

            }
        }
    ]



    confirm_visible: true;
    confirm_block: SELF.DBTable{state: "update"}
}
