/********************   License  English   *************************************
 *    Library: Data Registry Manager
 *    File: DrmUiWidget.h
 *    last change: Sergey Gershkovich, 2012/1/3
 *
 *
 *    Copyright (c) 2012, Sergey Gerskovich
 *    DaReManager.sourceforge.net
 *    SergeyGershkovich@gmail.com
 *    All rights reserved.
 *
 *
 *    Redistribution and use in source and binary forms, with or without
 *    modification, are permitted provided that the following conditions
 *    are met:
 *
 *         1. Redistributions of source code must retain the above copyright
 *            notice, this list of conditions and the following disclaimer.
 *         2. Redistributions in binary form must reproduce the above
 *            copyright notice, this list of conditions and the following
 *            disclaimer in the documentation and/or other materials provided
 *            with the distribution.
 *
 *    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 *    TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 *    OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 ********************   License Russian, UTF-8  ********************************
 *
 *    Библиотека: Управляющего реестром данных
 *    Файл: DrmUiWidget.h
 *
 *    Copyright (c) 2012, Сергей Гершкович
 *    DaReManager.sourceforge.net
 *    SergeyGershkovich@gmail.com
 *    Все права защищены.
 *
 *    Разрешается повторное распространение и использование как в виде
 *    исходного кода, так и в двоичной форме, с изменениями или без, при
 *    соблюдении следующих условий:
 *
 *        1. При повторном распространении исходного кода должно оставаться
 *           указанное выше уведомление об авторском праве, этот список условий
 *           и последующий отказ от гарантий.
 *        2. При повторном распространении двоичного кода должно сохраняться
 *           указанная выше информация об авторском праве, этот список условий
 *           и последующий отказ от гарантий в документации и/или в других
 *           материалах, поставляемых при распространении.
 *
 *    ЭТА ПРОГРАММА ПРЕДОСТАВЛЕНА БЕСПЛАТНО ВЛАДЕЛЬЦАМИ АВТОРСКИХ ПРАВ И/ИЛИ
 *    ДРУГИМИ СТОРОНАМИ "КАК ОНА ЕСТЬ" БЕЗ КАКОГО-ЛИБО ВИДА ГАРАНТИЙ,
 *    ВЫРАЖЕННЫХ ЯВНО ИЛИ ПОДРАЗУМЕВАЕМЫХ, ВКЛЮЧАЯ, НО НЕ ОГРАНИЧИВАЯСЬ ИМИ,
 *    ПОДРАЗУМЕВАЕМЫЕ ГАРАНТИИ КОММЕРЧЕСКОЙ ЦЕННОСТИ И ПРИГОДНОСТИ ДЛЯ
 *    КОНКРЕТНОЙ ЦЕЛИ. НИ В КОЕМ СЛУЧАЕ, ЕСЛИ НЕ ТРЕБУЕТСЯ СООТВЕТСТВУЮЩИМ
 *    ЗАКОНОМ, ИЛИ НЕ УСТАНОВЛЕНО В УСТНОЙ ФОРМЕ, НИ ОДИН ВЛАДЕЛЕЦ АВТОРСКИХ
 *    ПРАВ И НИ ОДНО ДРУГОЕ ЛИЦО, КОТОРОЕ МОЖЕТ ИЗМЕНЯТЬ И/ИЛИ ПОВТОРНО
 *    РАСПРОСТРАНЯТЬ ПРОГРАММУ, КАК БЫЛО СКАЗАНО ВЫШЕ, НЕ НЕСЁТ
 *    ОТВЕТСТВЕННОСТИ, ВКЛЮЧАЯ ЛЮБЫЕ ОБЩИЕ, СЛУЧАЙНЫЕ, СПЕЦИАЛЬНЫЕ ИЛИ
 *    ПОСЛЕДОВАВШИЕ УБЫТКИ, ВСЛЕДСТВИЕ ИСПОЛЬЗОВАНИЯ ИЛИ НЕВОЗМОЖНОСТИ
 *    ИСПОЛЬЗОВАНИЯ ПРОГРАММЫ (ВКЛЮЧАЯ, НО НЕ ОГРАНИЧИВАЯСЬ ПОТЕРЕЙ ДАННЫХ,
 *    ИЛИ ДАННЫМИ, СТАВШИМИ НЕПРАВИЛЬНЫМИ, ИЛИ ПОТЕРЯМИ ПРИНЕСЕННЫМИ ИЗ-ЗА ВАС
 *    ИЛИ ТРЕТЬИХ ЛИЦ, ИЛИ ОТКАЗОМ ПРОГРАММЫ РАБОТАТЬ СОВМЕСТНО С ДРУГИМИ
 *    ПРОГРАММАМИ), ДАЖЕ ЕСЛИ ТАКОЙ ВЛАДЕЛЕЦ ИЛИ ДРУГОЕ ЛИЦО БЫЛИ ИЗВЕЩЕНЫ О
 *    ВОЗМОЖНОСТИ ТАКИХ УБЫТКОВ.
 *
 */

#ifndef DRMUIWIDGET_H
#define DRMUIWIDGET_H

#include <QWidget>
#include <QGridLayout>
#include <qmath.h>

#include "../../sys/DrmSysDataSet.h"
#include "../../sys/DrmSysBlock.h"
#include "../../sys/DrmSysCascadeBlock.h"

///////////////////////////////////////////////////////////////////////////////
// Слово
///////////////////////////////////////////////////////////////////////////////
class DrmUiWidgetItem : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString field READ getField WRITE setField)
    Q_PROPERTY(bool visible READ getVisible WRITE setVisible)

    Q_PROPERTY(DrmSysBlock* reload_block READ getReloadBlock WRITE setReloadBlock)
public:
    void setField(QString Field){FieldPrivate=Field;}
    QString getField(){return FieldPrivate;}
    bool getVisible(){return VisiblePrivate;}

    void setReloadBlock(DrmSysBlock* Block);
    DrmSysBlock* getReloadBlock(){return ReloadBlockContainerPrivate.getLink();}
    void ReloadBlockStart();

    explicit DrmUiWidgetItem(QObject *parent = 0);
signals:
    void visibleChanged(bool );


    void takedValues(DrmSysDataSet*  DataSet);
    void returnedValues(DrmSysDataSet*  DataSet);

public slots:
    void setVisible(bool Visible);

    void ReloadBlockFinished(DrmSysDataSet *DataSet);
private:
    QString FieldPrivate;
    bool VisiblePrivate;

    bool isReloadBlock;
    DrmSysBlockContainer ReloadBlockContainerPrivate;
    QStandardItemModel ReloadBlockInPutDataSet;
};

class DrmUiWidgetItemBool : public DrmUiWidgetItem
{
    Q_OBJECT
    Q_PROPERTY(bool value READ getValue WRITE setValue)
public:
    bool getValue(){return ValuePrivate;}
signals:
    void valueChanged(bool );
public slots:
    void setValue(bool Value){if (ValuePrivate!=Value) {ValuePrivate=Value; valueChanged(Value); ReloadBlockStart();}}
private:
    bool ValuePrivate;
};

class DrmUiWidgetItemString : public DrmUiWidgetItem
{
    Q_OBJECT
    Q_PROPERTY(QString value READ getValue WRITE setValue)
public:
    QString getValue(){return ValuePrivate;}
signals:
    void valueChanged(QString );
public slots:
    void setValue(QString Value){if (ValuePrivate!=Value) {ValuePrivate=Value; valueChanged(Value);ReloadBlockStart();}}
private:
    QString ValuePrivate;
};

class DrmUiWidgetItemInteger : public DrmUiWidgetItem
{
    Q_OBJECT
    Q_PROPERTY(int value READ getValue WRITE setValue)
public:
    int getValue(){return ValuePrivate;}
signals:
    void valueChanged(int );
public slots:
    void setValue(int Value){if (ValuePrivate!=Value) {ValuePrivate=Value; valueChanged(Value);ReloadBlockStart();}}
private:
    int ValuePrivate;
};

class DrmUiWidgetItemNumber : public DrmUiWidgetItem
{
    Q_OBJECT
    Q_PROPERTY(double value READ getValue WRITE setValue)
    Q_PROPERTY(int decimals READ getDecimals WRITE setDecimals)
public:
    double getValue(){return ValuePrivate;}

    void setDecimals(int Decimals){DecimalsPrivate=Decimals;}
    int getDecimals(){return DecimalsPrivate;}
signals:
    void valueChanged(double );
public slots:
    void setValue(double Value){if (ValuePrivate!=Value) {ValuePrivate=Value; valueChanged(Value);ReloadBlockStart();}}
private:
    double ValuePrivate;
    int DecimalsPrivate;
};

class DrmUiWidgetItemDate : public DrmUiWidgetItem
{
    Q_OBJECT
    Q_PROPERTY(QDate value READ getValue WRITE setValue)
public:
    QDate getValue(){return ValuePrivate;}
signals:
    void valueChanged(QDate );
public slots:
    void setValue(QDate Value);
private:
    QDate ValuePrivate;
};

///////////////////////////////////////////////////////////////////////////////
// Предложение (изречение, реквизит)
///////////////////////////////////////////////////////////////////////////////
class DrmUiWidget : public QObject
{
    Q_OBJECT
    Q_PROPERTY(DrmUiWidgetItemBool* enabled READ getEnabled )
    Q_PROPERTY(DrmUiWidgetItemString* caption READ getCaption )
public:
    explicit DrmUiWidget(QObject *parent = 0);

    DrmUiWidgetItemBool* getEnabled(){return &EnabledPrivate;}
    DrmUiWidgetItemString* getCaption(){return &CaptionPrivate;}

    virtual void drawWidgets(QWidget* parent);
    virtual int  drawWidgets(QGridLayout* ParentLayout,  int row);

protected:
    DrmUiWidgetItemBool EnabledPrivate;
    DrmUiWidgetItemString CaptionPrivate;

    DrmSysDataSetContainer InPutValuesContainer;


signals:
    void takedValues(DrmSysDataSet*  DataSet);
    void returnedValues(DrmSysDataSet*  DataSet);

public slots:
    virtual void setValues(DrmSysDataSet* DataSet);
    virtual void getValues(DrmSysDataSet* DataSet);

};

#endif // DRMUIWIDGET_H
