/***************************   English   **************************************
 *    Library: Data Registry Manager
 *    File: DrmUiWidgetRegistry.cpp
 *    last change: Sergey Gershkovich, 2010/01/05
 *
 *
 *    Copyright (c) 2008-2010, Sergey Gerskovich
 *    DaReManager.sourceforge.net
 *    SergeyGershkovich@gmail.com
 *    All rights reserved.
 *
 *
 *    Redistribution and use in source and binary forms, with or without
 *    modification, are permitted provided that the following conditions
 *    are met:
 *
 *         1. Redistributions of source code must retain the above copyright
 *            notice, this list of conditions and the following disclaimer.
 *         2. Redistributions in binary form must reproduce the above
 *            copyright notice, this list of conditions and the following
 *            disclaimer in the documentation and/or other materials provided
 *            with the distribution.
 *
 *    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 *    TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 *    OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 ***************************   Russian, UTF-8  *********************************
 *
 *    Библиотека: Управляющего реестром данных
 *    Файл: DrmUiWidgetRegistry.cpp
 *
 *    Copyright (c) 2008-2010, Сергей Гершкович
 *    DaReManager.sourceforge.net
 *    SergeyGershkovich@gmail.com
 *    Все права защищены.
 *
 *    Разрешается повторное распространение и использование как в виде
 *    исходного кода, так и в двоичной форме, с изменениями или без, при
 *    соблюдении следующих условий:
 *
 *        1. При повторном распространении исходного кода должно оставаться
 *           указанное выше уведомление об авторском праве, этот список условий
 *           и последующий отказ от гарантий.
 *        2. При повторном распространении двоичного кода должно сохраняться
 *           указанная выше информация об авторском праве, этот список условий
 *           и последующий отказ от гарантий в документации и/или в других
 *           материалах, поставляемых при распространении.
 *
 *    ЭТА ПРОГРАММА ПРЕДОСТАВЛЕНА БЕСПЛАТНО ВЛАДЕЛЬЦАМИ АВТОРСКИХ ПРАВ И/ИЛИ
 *    ДРУГИМИ СТОРОНАМИ "КАК ОНА ЕСТЬ" БЕЗ КАКОГО-ЛИБО ВИДА ГАРАНТИЙ,
 *    ВЫРАЖЕННЫХ ЯВНО ИЛИ ПОДРАЗУМЕВАЕМЫХ, ВКЛЮЧАЯ, НО НЕ ОГРАНИЧИВАЯСЬ ИМИ,
 *    ПОДРАЗУМЕВАЕМЫЕ ГАРАНТИИ КОММЕРЧЕСКОЙ ЦЕННОСТИ И ПРИГОДНОСТИ ДЛЯ
 *    КОНКРЕТНОЙ ЦЕЛИ. НИ В КОЕМ СЛУЧАЕ, ЕСЛИ НЕ ТРЕБУЕТСЯ СООТВЕТСТВУЮЩИМ
 *    ЗАКОНОМ, ИЛИ НЕ УСТАНОВЛЕНО В УСТНОЙ ФОРМЕ, НИ ОДИН ВЛАДЕЛЕЦ АВТОРСКИХ
 *    ПРАВ И НИ ОДНО ДРУГОЕ ЛИЦО, КОТОРОЕ МОЖЕТ ИЗМЕНЯТЬ И/ИЛИ ПОВТОРНО
 *    РАСПРОСТРАНЯТЬ ПРОГРАММУ, КАК БЫЛО СКАЗАНО ВЫШЕ, НЕ НЕСЁТ
 *    ОТВЕТСТВЕННОСТИ, ВКЛЮЧАЯ ЛЮБЫЕ ОБЩИЕ, СЛУЧАЙНЫЕ, СПЕЦИАЛЬНЫЕ ИЛИ
 *    ПОСЛЕДОВАВШИЕ УБЫТКИ, ВСЛЕДСТВИЕ ИСПОЛЬЗОВАНИЯ ИЛИ НЕВОЗМОЖНОСТИ
 *    ИСПОЛЬЗОВАНИЯ ПРОГРАММЫ (ВКЛЮЧАЯ, НО НЕ ОГРАНИЧИВАЯСЬ ПОТЕРЕЙ ДАННЫХ,
 *    ИЛИ ДАННЫМИ, СТАВШИМИ НЕПРАВИЛЬНЫМИ, ИЛИ ПОТЕРЯМИ ПРИНЕСЕННЫМИ ИЗ-ЗА ВАС
 *    ИЛИ ТРЕТЬИХ ЛИЦ, ИЛИ ОТКАЗОМ ПРОГРАММЫ РАБОТАТЬ СОВМЕСТНО С ДРУГИМИ
 *    ПРОГРАММАМИ), ДАЖЕ ЕСЛИ ТАКОЙ ВЛАДЕЛЕЦ ИЛИ ДРУГОЕ ЛИЦО БЫЛИ ИЗВЕЩЕНЫ О
 *    ВОЗМОЖНОСТИ ТАКИХ УБЫТКОВ.
 *
 */





#include "DrmUiWidgetRegistry.h"
#include "../../sys/DrmSysCascadeBlock.h"



DrmUiWidgetRegistry::DrmUiWidgetRegistry(QObject *parent)
    : DrmUiWidget(parent)
{
    SelectionRecNo=0;

    ReloadShortName="Reload";
    isReloadBlock=true;
    connect(ReloadBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)),SLOT(setAfterReload(DrmSysDataSet*)));

    TableViewModel.setSourceModel(&ReloadModelEmptyDataSet);


    SearchShortName="Search";
    SearchOn=false;

    isAddBlock=false;
    AddAndOpen=false;
    AddShortName="Add";

    isOpenBlock=false;
    OpenShortName="Open";

    isDeleteBlock=false;
    DeleteShortName="Delete";


}

void
DrmUiWidgetRegistry::recalculateFooterValues()
{
    for (int column=0; column< Columns.count(); ++column){

        QString Function=Columns.at(column)->getAggFunction();

        if (Function=="count") Columns.at(column)->setAggValue(TableViewModel.rowCount());
        else if (Function=="sum"){
            double sum=0;
            for (int row=0; row< TableViewModel.rowCount(); ++row)
                    sum+=TableViewModel.data(TableViewModel.index(row,column)).toDouble();

            Columns.at(column)->setAggValue(sum);
        }
    }
}

void
DrmUiWidgetRegistry::drawWidgets(QWidget *MainWidget)
{
    QVBoxLayout* MainLayout = new QVBoxLayout();
        MainLayout->setMargin(0);
        MainWidget->setLayout(MainLayout);

/* Params  */
        //DrmSysBlock* ParamsBlock=ParamsBlockContainer.getLink();

/* TopLayout  */
        QHBoxLayout* TopLayout = new QHBoxLayout();
            MainLayout->addLayout(TopLayout);


/* Reload  */
        QPushButton* ReloadButton = new QPushButton(ReloadShortName, MainWidget);
            TopLayout->addWidget(ReloadButton);
            ReloadButton->setVisible(isReloadBlock);
            connect(ReloadButton,SIGNAL(clicked()),SLOT(reload()));

/* Search  */
        QPushButton* SearchButton = new QPushButton(SearchShortName, MainWidget);
           TopLayout->addWidget(SearchButton);
           SearchButton->setVisible(isReloadBlock);
           SearchButton->setCheckable( true);
           SearchButton->setChecked(getSearchOn());



/* Add  */
        QPushButton* AddButton = new QPushButton(AddShortName, MainWidget);
            TopLayout->addWidget(AddButton);
            AddButton->setVisible(isAddBlock);
            connect(AddButton,SIGNAL(clicked()),SLOT(addNew()));

/* Open  */
        QPushButton* OpenButton = new QPushButton(OpenShortName, MainWidget);
            TopLayout->addWidget(OpenButton);
            OpenButton->setVisible(isOpenBlock);
            connect(OpenButton,SIGNAL(clicked()),SLOT(openSelected()));

/* Delete  */
        QPushButton* DeleteButton = new QPushButton(DeleteShortName, MainWidget);
            DeleteButton->setVisible(isDeleteBlock);
            TopLayout->addWidget(DeleteButton);
            connect(DeleteButton,SIGNAL(clicked()),SLOT(deleteSelected()));

         // натянуть пустующее пространство
         TopLayout->addStretch();



/* TableView  */

        TableViewModel.setFieldsOn(Columns.count()>0);
        if ((Columns.count()>0) && (TableViewModel.getFieldsList()->count()==0))
        {
            for (int column=0; column< Columns.count(); ++column)
            {
                TableViewModel.getFieldsList()->append(new DrmSysReplaceFieldsDataSetField(&TableViewModel));
                TableViewModel.getFieldsList()->last()->setCode(Columns.at(column)->getShortName());
                TableViewModel.getFieldsList()->last()->setSourceCode(Columns.at(column)->getCode());
            }
        }

         QTableView* TableView = new QTableView(MainWidget);
             TableView->setSortingEnabled(true);
             MainLayout->addWidget (TableView) ;
             TableView->connect(ReloadBlockContainer.getLink(), SIGNAL(finished(DrmSysDataSet*)),SLOT(resizeColumnsToContents()));

             TableView->setModel(&TableViewModel);

             QItemSelectionModel* Selection=new QItemSelectionModel(&TableViewModel,TableView);
             TableView->setSelectionModel(Selection);
             Selection->connect(this,SIGNAL(setCurrentIndex(QModelIndex,QItemSelectionModel::SelectionFlags)),SLOT(setCurrentIndex(QModelIndex,QItemSelectionModel::SelectionFlags)));
             connect(Selection,SIGNAL(selectionChanged ( const QItemSelection &, const QItemSelection &  )),SLOT(selectionChanged ( const QItemSelection &, const QItemSelection &  )));


/* FooterLayout  */
         QGridLayout* FooterLayout = new QGridLayout();
            MainLayout->addLayout(FooterLayout);

         for (int column=0; column< Columns.count(); ++column)
            if (!Columns.at(column)->getAggFunction().isEmpty()){
                QHBoxLayout* layout= new QHBoxLayout();
                    FooterLayout->addLayout(layout,column,0);
                    layout->addStretch();

                QLabel *lable = new QLabel(Columns.at(column)->getAggCaption(),MainWidget);
                layout->addWidget(lable,0,Qt::AlignRight);

                QLabel *lableagg = new QLabel("...",MainWidget);
                    QFont font;
                    font.setBold(true);
                    lableagg->setFont(font);
                    FooterLayout->addWidget(lableagg,column,1,Qt::AlignRight);
                    lableagg->connect(Columns.at(column),SIGNAL(recalculatedAggValue(QString)),SLOT(setText(QString)));
            }



/* SelectLayout  */
         QHBoxLayout* SelectLayout = new QHBoxLayout();
             MainLayout->addLayout(SelectLayout);

             QLabel* SelectLabel = new QLabel(SearchShortName, MainWidget);
             SelectLabel->setVisible(SearchButton->isChecked());
             SelectLabel->connect(SearchButton,SIGNAL(toggled(bool)),SLOT(setVisible(bool)));
             SelectLayout->addWidget(SelectLabel);


             QLineEdit* SelectEdit  = new QLineEdit(MainWidget);
             SelectEdit->setVisible(SearchButton->isChecked());
             SelectEdit->connect(SearchButton,SIGNAL(toggled(bool)),SLOT(setVisible(bool)));
             SelectLayout->addWidget(SelectEdit);
             connect(SelectEdit, SIGNAL(textChanged(QString)), SLOT(setSearchString(QString)));

}

void
DrmUiWidgetRegistry::selectionChanged ( const QItemSelection & selected, const QItemSelection &  )
{
    if (!selected.isEmpty()) SelectionRecNo = selected.at(0).topLeft().row();

   for (int i=TableViewModel.columnCount()-1; 0<=i; --i)
      setCurrentIndex(TableViewModel.index(SelectionRecNo,i),QItemSelectionModel::Select);
}


void
DrmUiWidgetRegistry::setCurrentRow (DrmSysDataSet* DataSet)
{
    return; if (DataSet->rowCount()<=0) return;

    DrmSysDataSet* ReloadDataSet=TableViewModel.sourceModel();

    for (int row=0; row<TableViewModel.rowCount(); ++row)
    {
        bool FindedRow=true;
        for (int column=0; column<DataSet->columnCount() && FindedRow; ++column)
        {
            QString FieldCode=DataSet->headerData(column, Qt::Horizontal).toString();
            QVariant FieldValue=DataSet->data(DataSet->index(0,column));

            if (FieldValue!=DrmSysDataSet_getFieldValue(ReloadDataSet, FieldCode, FieldValue, row))
                      FindedRow=false;
        }

        if (FindedRow) {
                setCurrentIndex(TableViewModel.index(row,0),QItemSelectionModel::Select);

            return;
        }

    }
}

void
DrmUiWidgetRegistry::setValues(DrmSysDataSet* DataSet)
{
    bool isReload=false;
    foreach (QString field, InputFields){
        QVariant oldValue=DrmSysDataSet_getFieldValue(&ReloadBlockInPutDataSet, field);
        QVariant newValue=DrmSysDataSet_getFieldValue(DataSet, field, oldValue);
        if (oldValue!=newValue)
        {
            isReload=true;
        }
    }

    DrmSysDataSet_widen(DataSet,&ReloadBlockInPutDataSet);

    if (isReload || InputFields.count()==0) reload();
}

void
DrmUiWidgetRegistry::getValues(DrmSysDataSet* DataSet)
{
    if (TableViewModel.rowCount()==0) return;

    if (SelectionRecNo>=TableViewModel.rowCount()) SelectionRecNo=TableViewModel.rowCount()-1;
    if (SelectionRecNo<0) SelectionRecNo=0;


    int row=TableViewModel.mapToSource(TableViewModel.index(SelectionRecNo,0)).row();

    QString FieldName;
    for (int column=0; column<TableViewModel.sourceModel()->columnCount(); ++column)
    {
         FieldName = TableViewModel.sourceModel()->headerData(column, Qt::Horizontal).toString();
         DrmSysDataSet_setFieldValue(DataSet
                                     , FieldName
                                     , TableViewModel.sourceModel()->data(TableViewModel.sourceModel()->index(row, column))
                                     //, i
                                     );
    }
}


void
DrmUiWidgetRegistry::setReloadBlock(DrmSysBlock* Block)
{
    disconnect(ReloadBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)), this, SLOT(setAfterReload(DrmSysDataSet*)));
    ReloadBlockContainer.setLink(Block);
    connect(ReloadBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)),SLOT(setAfterReload(DrmSysDataSet*)));


    isReloadBlock=true;
}


void
DrmUiWidgetRegistry::reload()
{
    TableViewModel.setSourceModel(&ReloadModelEmptyDataSet);

    takedValues(&ReloadBlockInPutDataSet);

    ReloadBlockContainer.getLink()->start(&ReloadBlockInPutDataSet);

}
void
DrmUiWidgetRegistry::setAfterReload(DrmSysDataSet* DataSet)
{
    TableViewModel.setSourceModel(DataSet);
    recalculateFooterValues();
}

void
DrmUiWidgetRegistry::setSearchString(QString String)
{
    TableViewModel.FilterWords.clear();

    if (!String.isEmpty()){
        TableViewModel.FilterWords = String.split(QRegExp("\\s+"));
    }


    TableViewModel.refresh();
    recalculateFooterValues();
}


void
DrmUiWidgetRegistry::setAddBlock(DrmSysBlock* Block)
{
    disconnect(AddBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)), this, SLOT(setAfterAddNew(DrmSysDataSet*)));
    AddBlockContainer.setLink(Block);
    connect(AddBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)),SLOT(setAfterAddNew(DrmSysDataSet*)));

    isAddBlock=true;
}
void
DrmUiWidgetRegistry::addNew()
{
    AddBlockInPutDataSet.clear();
    for (int column=0; column<TableViewModel.sourceModel()->columnCount(); ++column)
    {
        DrmSysDataSet_setFieldValue(&AddBlockInPutDataSet
                                    , TableViewModel.sourceModel()->headerData(column, Qt::Horizontal).toString()
                                    , QVariant());
    }

    AddBlockContainer.getLink()->start(&AddBlockInPutDataSet);
}
void
DrmUiWidgetRegistry::setAfterAddNew(DrmSysDataSet* DataSet)
{
    if (AddAndOpen)
        OpenBlockContainer.getLink()->start(DataSet);
    else{
        reload();
        setCurrentRow(DataSet);
    }
}

void
DrmUiWidgetRegistry::setOpenBlock(DrmSysBlock* Block)
{
    disconnect(OpenBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)), this, SLOT(setAfterOpenSelected(DrmSysDataSet*)));
    OpenBlockContainer.setLink(Block);
    connect(OpenBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)),SLOT(setAfterOpenSelected(DrmSysDataSet*)));

    isOpenBlock=true;
}
void
DrmUiWidgetRegistry::openSelected()
{
    getValues(&OpenBlockInPutDataSet);
    OpenBlockContainer.getLink()->start(&OpenBlockInPutDataSet);
}
void
DrmUiWidgetRegistry::setAfterOpenSelected(DrmSysDataSet* DataSet)
{
    reload();
    setCurrentRow (DataSet);
}

void
DrmUiWidgetRegistry::setDeleteBlock(DrmSysBlock* Block)
{
    disconnect(DeleteBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)), this, SLOT(setAfterDeleteSelected(DrmSysDataSet*)));
    DeleteBlockContainer.setLink(Block);
    connect(DeleteBlockContainer.getLink(),SIGNAL(finished(DrmSysDataSet*)),SLOT(setAfterDeleteSelected(DrmSysDataSet*)));

    isDeleteBlock=true;
}
void
DrmUiWidgetRegistry::deleteSelected()
{
    getValues(&DeleteBlockInPutDataSet);
    DeleteBlockContainer.getLink()->start(&DeleteBlockInPutDataSet);
}
void
DrmUiWidgetRegistry::setAfterDeleteSelected(DrmSysDataSet* DataSet)
{
    reload();
    setCurrentRow (DataSet);

}

