/***************************   English   **************************************
 *    Library: Data Registry Manager
 *    File: DrmUiWidgetRegistry.h
 *    last change: Sergey Gershkovich, 2009/12/22
 *
 *
 *    Copyright (c) 2008-2009, Sergey Gerskovich
 *    DaReManager.sourceforge.net
 *    SergeyGershkovich@gmail.com
 *    All rights reserved.
 *
 *
 *    Redistribution and use in source and binary forms, with or without
 *    modification, are permitted provided that the following conditions
 *    are met:
 *
 *         1. Redistributions of source code must retain the above copyright
 *            notice, this list of conditions and the following disclaimer.
 *         2. Redistributions in binary form must reproduce the above
 *            copyright notice, this list of conditions and the following
 *            disclaimer in the documentation and/or other materials provided
 *            with the distribution.
 *
 *    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 *    TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 *    OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 ***************************   Russian, UTF-8  *********************************
 *
 *    Библиотека: Управляющего реестром данных
 *    Файл: DrmUiWidgetRegistry.h
 *
 *    Copyright (c) 2008-2009, Сергей Гершкович
 *    DaReManager.sourceforge.net
 *    SergeyGershkovich@gmail.com
 *    Все права защищены.
 *
 *    Разрешается повторное распространение и использование как в виде
 *    исходного кода, так и в двоичной форме, с изменениями или без, при
 *    соблюдении следующих условий:
 *
 *        1. При повторном распространении исходного кода должно оставаться
 *           указанное выше уведомление об авторском праве, этот список условий
 *           и последующий отказ от гарантий.
 *        2. При повторном распространении двоичного кода должно сохраняться
 *           указанная выше информация об авторском праве, этот список условий
 *           и последующий отказ от гарантий в документации и/или в других
 *           материалах, поставляемых при распространении.
 *
 *    ЭТА ПРОГРАММА ПРЕДОСТАВЛЕНА БЕСПЛАТНО ВЛАДЕЛЬЦАМИ АВТОРСКИХ ПРАВ И/ИЛИ
 *    ДРУГИМИ СТОРОНАМИ "КАК ОНА ЕСТЬ" БЕЗ КАКОГО-ЛИБО ВИДА ГАРАНТИЙ,
 *    ВЫРАЖЕННЫХ ЯВНО ИЛИ ПОДРАЗУМЕВАЕМЫХ, ВКЛЮЧАЯ, НО НЕ ОГРАНИЧИВАЯСЬ ИМИ,
 *    ПОДРАЗУМЕВАЕМЫЕ ГАРАНТИИ КОММЕРЧЕСКОЙ ЦЕННОСТИ И ПРИГОДНОСТИ ДЛЯ
 *    КОНКРЕТНОЙ ЦЕЛИ. НИ В КОЕМ СЛУЧАЕ, ЕСЛИ НЕ ТРЕБУЕТСЯ СООТВЕТСТВУЮЩИМ
 *    ЗАКОНОМ, ИЛИ НЕ УСТАНОВЛЕНО В УСТНОЙ ФОРМЕ, НИ ОДИН ВЛАДЕЛЕЦ АВТОРСКИХ
 *    ПРАВ И НИ ОДНО ДРУГОЕ ЛИЦО, КОТОРОЕ МОЖЕТ ИЗМЕНЯТЬ И/ИЛИ ПОВТОРНО
 *    РАСПРОСТРАНЯТЬ ПРОГРАММУ, КАК БЫЛО СКАЗАНО ВЫШЕ, НЕ НЕСЁТ
 *    ОТВЕТСТВЕННОСТИ, ВКЛЮЧАЯ ЛЮБЫЕ ОБЩИЕ, СЛУЧАЙНЫЕ, СПЕЦИАЛЬНЫЕ ИЛИ
 *    ПОСЛЕДОВАВШИЕ УБЫТКИ, ВСЛЕДСТВИЕ ИСПОЛЬЗОВАНИЯ ИЛИ НЕВОЗМОЖНОСТИ
 *    ИСПОЛЬЗОВАНИЯ ПРОГРАММЫ (ВКЛЮЧАЯ, НО НЕ ОГРАНИЧИВАЯСЬ ПОТЕРЕЙ ДАННЫХ,
 *    ИЛИ ДАННЫМИ, СТАВШИМИ НЕПРАВИЛЬНЫМИ, ИЛИ ПОТЕРЯМИ ПРИНЕСЕННЫМИ ИЗ-ЗА ВАС
 *    ИЛИ ТРЕТЬИХ ЛИЦ, ИЛИ ОТКАЗОМ ПРОГРАММЫ РАБОТАТЬ СОВМЕСТНО С ДРУГИМИ
 *    ПРОГРАММАМИ), ДАЖЕ ЕСЛИ ТАКОЙ ВЛАДЕЛЕЦ ИЛИ ДРУГОЕ ЛИЦО БЫЛИ ИЗВЕЩЕНЫ О
 *    ВОЗМОЖНОСТИ ТАКИХ УБЫТКОВ.
 *
 */


#ifndef DRMUIWIDGETREGISTRY_H
#define DRMUIWIDGETREGISTRY_H

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QBoxLayout>
#include <QPushButton>
#include <QTableView>
#include <QItemSelectionModel>
#include <QStandardItemModel>

#include "DrmUiWidget.h"
#include "../../sys/DrmSysDataSet.h"
#include "../../sys/DrmSysBlock.h"


class DrmUiWidgetRegistryColumn : public DrmSysObject
{
    Q_OBJECT
    Q_PROPERTY(QString agg_function READ getAggFunction WRITE setAggFunction)
    Q_PROPERTY(QString agg_caption READ getAggCaption WRITE setAggCaption)
public:
    QString getAggFunction(){return AggFunctionPrivate;}
    void setAggFunction(QString AggFunction){AggFunctionPrivate=AggFunction;}

    QString getAggCaption(){if (AggCaptionPrivate.isEmpty()) return getFullName(); else return AggCaptionPrivate;}
    void setAggCaption(QString AggCaption){AggCaptionPrivate=AggCaption;}

    void setAggValue(QVariant Value){recalculatedAggValue(QLocale().toString(Value.toDouble(), 'f', 2));}

private:
    QString AggFunctionPrivate;
    QString AggCaptionPrivate;

signals:
    void recalculatedAggValue(QString Value);

};

class DrmUiWidgetRegistryColumnNumder : public DrmUiWidgetRegistryColumn
{
    Q_OBJECT
    Q_PROPERTY(int decimals READ getDecimals WRITE setDecimals)
public:
    int getDecimals(){return Decimals;}
    void setDecimals(int Decimals){this->Decimals=Decimals;}
private:
    int Decimals;
};

class DrmUiWidgetRegistry : public DrmUiWidget
{
    Q_OBJECT

    Q_PROPERTY(QStringList input_fields READ getInputFields WRITE setInputFields)

    Q_PROPERTY(QDeclarativeListProperty <DrmUiWidgetRegistryColumn> columns READ getColumns)

    Q_PROPERTY(DrmSysBlock* reload_block READ getReloadBlock WRITE setReloadBlock)
    Q_PROPERTY(QString reload_shortname READ getReloadShortName WRITE setReloadShortName)

    Q_PROPERTY(QString search_shortname READ getSearchShortName WRITE setSearchShortName)
    Q_PROPERTY(bool search_on READ getSearchOn WRITE setSearchOn)

    Q_PROPERTY(DrmSysBlock* add_block READ getAddBlock WRITE setAddBlock)
    Q_PROPERTY(bool add_and_open READ getAddAndOpen WRITE setAddAndOpen)
    Q_PROPERTY(QString add_shortname READ getAddShortName WRITE setAddShortName)

    Q_PROPERTY(DrmSysBlock* open_block READ getOpenBlock WRITE setOpenBlock)
    Q_PROPERTY(QString open_shortname READ getOpenShortName WRITE setOpenShortName)

    Q_PROPERTY(DrmSysBlock* delete_block READ getDeleteBlock WRITE setDeleteBlock)
    Q_PROPERTY(QString delete_shortname READ getDeleteShortName WRITE setDeleteShortName)

private:
    QStringList InputFields;

    QList< DrmUiWidgetRegistryColumn* > Columns;

    DrmSysBlockContainer ParamsBlockContainer;

    QString ReloadShortName;
    DrmSysBlockContainer ReloadBlockContainer;
    QStandardItemModel ReloadBlockInPutDataSet;
    DrmSysReplaceFieldsDataSet TableViewModel;
    QStandardItemModel ReloadModelEmptyDataSet;
    bool isReloadBlock;

    QString SearchShortName;
    bool SearchOn;

    QString AddShortName;
    bool AddAndOpen;
    DrmSysBlockContainer AddBlockContainer;
    QStandardItemModel AddBlockInPutDataSet;
    bool isAddBlock;

    QString OpenShortName;
    DrmSysBlockContainer OpenBlockContainer;
    QStandardItemModel OpenBlockInPutDataSet;
    bool isOpenBlock;

    QString DeleteShortName;
    DrmSysBlockContainer DeleteBlockContainer;
    QStandardItemModel DeleteBlockInPutDataSet;
    bool isDeleteBlock;

    int SelectionRecNo;

    void recalculateFooterValues();

public:
    explicit DrmUiWidgetRegistry(QObject *parent = 0);

    QStringList getInputFields() {return InputFields;}
    void setInputFields(QStringList Fields) {InputFields=Fields;}

    QDeclarativeListProperty <DrmUiWidgetRegistryColumn> getColumns() {return QDeclarativeListProperty<DrmUiWidgetRegistryColumn>(this, Columns);}

    virtual void getValues(DrmSysDataSet* DataSet);
    virtual void setValues(DrmSysDataSet* DataSet);

    DrmSysBlock* getParamsBlock(){return ParamsBlockContainer.getLink();}
    void setParamsBlock(DrmSysBlock* Block){ParamsBlockContainer.setLink(Block);}

    DrmSysBlock* getReloadBlock(){return ReloadBlockContainer.getLink();}
    void setReloadBlock(DrmSysBlock* Block);

    QString getReloadShortName(){return ReloadShortName;}
    void setReloadShortName(QString Name) {ReloadShortName=Name;}

    QString getSearchShortName(){return SearchShortName;}
    void setSearchShortName(QString Name) {SearchShortName=Name;}
    bool getSearchOn(){return SearchOn;}
    void setSearchOn(bool On) {SearchOn=On;}

    DrmSysBlock* getAddBlock(){return AddBlockContainer.getLink();}
    void setAddBlock(DrmSysBlock* Block);
    QString getAddShortName(){return AddShortName;}
    void setAddShortName(QString Name) {AddShortName=Name;}
    bool getAddAndOpen(){return AddAndOpen;}
    void setAddAndOpen(bool AddAndOpen){this->AddAndOpen=AddAndOpen;}

    DrmSysBlock* getOpenBlock(){return OpenBlockContainer.getLink();}
    void setOpenBlock(DrmSysBlock* Block);
    QString getOpenShortName(){return OpenShortName;}
    void setOpenShortName(QString Name) {OpenShortName=Name;}

    DrmSysBlock* getDeleteBlock(){return DeleteBlockContainer.getLink();}
    void setDeleteBlock(DrmSysBlock* Block);
    QString getDeleteShortName(){return DeleteShortName;}
    void setDeleteShortName(QString Name) {DeleteShortName=Name;}

    void drawWidgets(QWidget *MainWidget);

signals:
    void setCurrentIndex ( const QModelIndex & index, QItemSelectionModel::SelectionFlags command );

public slots:
    void reload();
    void setAfterReload(DrmSysDataSet* DataSet);

    void setSearchString(QString String);

    void addNew();
    void setAfterAddNew(DrmSysDataSet* DataSet);

    void selectionChanged ( const QItemSelection & selected, const QItemSelection &  );
    void setCurrentRow (DrmSysDataSet* DataSet);

    void openSelected();
    void setAfterOpenSelected(DrmSysDataSet* DataSet);

    void deleteSelected();
    void setAfterDeleteSelected(DrmSysDataSet* DataSet);

};

#endif // DRMUIWIDGETREGISTRY_H
